"""DSM Information data."""


class SynoDSMInformation:
    """Class containing Information data."""

    API_KEY = "SYNO.DSM.Info"

    def __init__(self, dsm):
        """Constructor methods."""
        self._dsm = dsm
        self._data = {}

    def update(self):
        """Updates information data."""
        raw_data = self._dsm.get(self.API_KEY, "getinfo")
        if raw_data:
            self._data = raw_data["data"]

    @property
    def model(self):
        """Model of the NAS."""
        return self._data.get("model")

    @property
    def ram(self):
        """RAM of the NAS (in MB)."""
        return self._data.get("ram")

    @property
    def serial(self):
        """Serial of the NAS."""
        return self._data.get("serial")

    @property
    def temperature(self):
        """Temperature of the NAS."""
        return self._data.get("temperature")

    @property
    def temperature_warn(self):
        """Temperature warning of the NAS."""
        return self._data.get("temperature_warn")

    @property
    def uptime(self):
        """Uptime of the NAS."""
        return self._data.get("uptime")

    @property
    def version(self):
        """Version of the NAS (build version)."""
        return self._data.get("version")

    @property
    def version_string(self):
        """Version of the NAS."""
        return self._data.get("version_string")



"""DSM Utilization data."""
from synology_dsm.helpers import SynoFormatHelper


class SynoCoreUtilization:
    """Class containing Utilization data."""

    API_KEY = "SYNO.Core.System.Utilization"

    def __init__(self, dsm):
        """Constructor method."""
        self._dsm = dsm
        self._data = {}

    def update(self):
        """Updates utilization data."""
        raw_data = self._dsm.get(self.API_KEY, "get")
        if raw_data:
            self._data = raw_data["data"]

    @property
    def cpu(self):
        """Gets CPU utilization."""
        return self._data.get("cpu", {})

    @property
    def cpu_other_load(self):
        """Other percentage of the total CPU load."""
        return self.cpu.get("other_load")

    @property
    def cpu_user_load(self):
        """User percentage of the total CPU load."""
        return self.cpu.get("user_load")

    @property
    def cpu_system_load(self):
        """System percentage of the total CPU load."""
        return self.cpu.get("system_load")

    @property
    def cpu_total_load(self):
        """Total CPU load for Synology DSM."""
        system_load = self.cpu_system_load
        user_load = self.cpu_user_load
        other_load = self.cpu_other_load

        if system_load is not None and user_load is not None and other_load is not None:
            return system_load + user_load + other_load
        return None

    @property
    def cpu_1min_load(self):
        """Average CPU load past minute."""
        return self.cpu.get("1min_load")

    @property
    def cpu_5min_load(self):
        """Average CPU load past 5 minutes."""
        return self.cpu.get("5min_load")

    @property
    def cpu_15min_load(self):
        """Average CPU load past 15 minutes."""
        return self.cpu.get("15min_load")

    @property
    def memory(self):
        """Gets memory utilization."""
        return self._data.get("memory")

    @property
    def memory_real_usage(self):
        """Real Memory usage from Synology DSM."""
        if self.memory:
            return str(self._data["memory"]["real_usage"])
        return None

    def memory_size(self, human_readable=False):
        """Total memory size of Synology DSM."""
        if self.memory:
            # Memory is actually returned in KB's so multiply before converting
            return_data = int(self._data["memory"]["memory_size"]) * 1024
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None

    def memory_available_swap(self, human_readable=False):
        """Total available memory swap."""
        if self.memory:
            # Memory is actually returned in KB's so multiply before converting
            return_data = int(self._data["memory"]["avail_swap"]) * 1024
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None

    def memory_cached(self, human_readable=False):
        """Total cached memory."""
        if self.memory:
            # Memory is actually returned in KB's so multiply before converting
            return_data = int(self._data["memory"]["cached"]) * 1024
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None

    def memory_available_real(self, human_readable=False):
        """Real available memory."""
        if self.memory:
            # Memory is actually returned in KB's so multiply before converting
            return_data = int(self._data["memory"]["avail_real"]) * 1024
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None

    def memory_total_real(self, human_readable=False):
        """Total available real memory."""
        if self.memory:
            # Memory is actually returned in KB's so multiply before converting
            return_data = int(self._data["memory"]["total_real"]) * 1024
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None

    def memory_total_swap(self, human_readable=False):
        """Total swap memory."""
        if self.memory:
            # Memory is actually returned in KB's so multiply before converting
            return_data = int(self._data["memory"]["total_swap"]) * 1024
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None

    @property
    def network(self):
        """Gets network utilization."""
        return self._data.get("network", [])

    def _get_network(self, network_id):
        """Function to get specific network (eth0, total, etc)."""
        for network in self.network:
            if network["device"] == network_id:
                return network
        return None

    def network_up(self, human_readable=False):
        """Total upload speed being used."""
        network = self._get_network("total")
        if network:
            return_data = int(network["tx"])
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None

    def network_down(self, human_readable=False):
        """Total download speed being used."""
        network = self._get_network("total")
        if network:
            return_data = int(network["rx"])
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None



"""DownloadStation task."""


class SynoDownloadTask:
    """An representation of a Synology DownloadStation task."""

    def __init__(self, data):
        """Initialize a Download Station task."""
        self._data = data

    def update(self, data):
        """Update the task."""
        self._data = data

    @property
    def id(self):
        """Return id of the task."""
        return self._data["id"]

    @property
    def title(self):
        """Return title of the task."""
        return self._data["title"]

    @property
    def type(self):
        """Return type of the task (bt, nzb, http(s), ftp, emule)."""
        return self._data["type"]

    @property
    def username(self):
        """Return username of the task."""
        return self._data["username"]

    @property
    def size(self):
        """Return size of the task."""
        return self._data["size"]

    @property
    def status(self):
        """Return status of the task.

        Possible values: waiting, downloading, paused, finishing, finished,
            hash_checking, seeding, filehosting_waiting, extracting, error
        """
        return self._data["status"]

    @property
    def status_extra(self):
        """Return status_extra of the task."""
        return self._data.get("status_extra")

    @property
    def additional(self):
        """Return additional data of the task."""
        return self._data["additional"]



"""Shared Folders data."""
from synology_dsm.helpers import SynoFormatHelper


class SynoCoreShare:
    """Class containing Share data."""

    API_KEY = "SYNO.Core.Share"
    # Syno supports two methods to retrieve resource details, GET and POST.
    # GET returns a limited set of keys. With POST the same keys as GET
    # are returned plus any keys listed in the "additional" parameter.
    # NOTE: The value of the additional key must be a string.
    REQUEST_DATA = {
        "additional": '["hidden","encryption","is_aclmode","unite_permission",'
        '"is_support_acl","is_sync_share","is_force_readonly","force_readonly_reason",'
        '"recyclebin","is_share_moving","is_cluster_share","is_exfat_share",'
        '"is_cold_storage_share","support_snapshot","share_quota",'
        '"enable_share_compress","enable_share_cow","include_cold_storage_share",'
        '"is_cold_storage_share"]',
        "shareType": "all",
    }

    def __init__(self, dsm):
        """Constructor method."""
        self._dsm = dsm
        self._data = {}

    def update(self):
        """Updates share data."""
        raw_data = self._dsm.post(self.API_KEY, "list", data=self.REQUEST_DATA)
        if raw_data:
            self._data = raw_data["data"]

    @property
    def shares(self):
        """Gets all shares."""
        return self._data.get("shares", [])

    @property
    def shares_uuids(self):
        """Return (internal) share ids."""
        shares = []
        for share in self.shares:
            shares.append(share["uuid"])
        return shares

    def get_share(self, share_uuid):
        """Returns a specific share by uuid.."""
        for share in self.shares:
            if share["uuid"] == share_uuid:
                return share
        return {}

    def share_name(self, share_uuid):
        """Return the name of this share."""
        return self.get_share(share_uuid).get("name")

    def share_path(self, share_uuid):
        """Return the volume path of this share."""
        return self.get_share(share_uuid).get("vol_path")

    def share_recycle_bin(self, share_uuid):
        """Is the recycle bin enabled for this share?"""
        return self.get_share(share_uuid).get("enable_recycle_bin")

    def share_size(self, share_uuid, human_readable=False):
        """Total size of share."""
        share_size_mb = self.get_share(share_uuid).get("share_quota_used")
        # Share size is returned in MB so we convert it.
        share_size_bytes = SynoFormatHelper.megabytes_to_bytes(share_size_mb)
        if human_readable:
            return SynoFormatHelper.bytes_to_readable(share_size_bytes)
        return share_size_bytes



"""Synology SurveillanceStation API wrapper."""
from .camera import SynoCamera
from .const import MOTION_DETECTION_BY_SURVEILLANCE
from .const import MOTION_DETECTION_DISABLED
from .const import SNAPSHOT_PROFILE_BALANCED


class SynoSurveillanceStation:
    """An implementation of a Synology SurveillanceStation."""

    API_KEY = "SYNO.SurveillanceStation.*"
    INFO_API_KEY = "SYNO.SurveillanceStation.Info"
    CAMERA_API_KEY = "SYNO.SurveillanceStation.Camera"
    CAMERA_EVENT_API_KEY = "SYNO.SurveillanceStation.Camera.Event"
    HOME_MODE_API_KEY = "SYNO.SurveillanceStation.HomeMode"
    SNAPSHOT_API_KEY = "SYNO.SurveillanceStation.SnapShot"

    def __init__(self, dsm):
        """Initialize a Surveillance Station."""
        self._dsm = dsm
        self._cameras_by_id = {}

    def update(self):
        """Update cameras and motion settings with latest from API."""
        self._cameras_by_id = {}
        list_data = self._dsm.get(self.CAMERA_API_KEY, "List", max_version=7)["data"]
        for camera_data in list_data["cameras"]:
            if camera_data["id"] in self._cameras_by_id:
                self._cameras_by_id[camera_data["id"]].update(camera_data)
            else:
                self._cameras_by_id[camera_data["id"]] = SynoCamera(camera_data)

        for camera_id in self._cameras_by_id:
            self._cameras_by_id[camera_id].update_motion_detection(
                self._dsm.get(
                    self.CAMERA_EVENT_API_KEY, "MotionEnum", {"camId": camera_id}
                )["data"]
            )

        if not self._cameras_by_id:
            return

        live_view_datas = self._dsm.get(
            self.CAMERA_API_KEY,
            "GetLiveViewPath",
            {"idList": ",".join(str(k) for k in self._cameras_by_id)},
        )["data"]
        for live_view_data in live_view_datas:
            self._cameras_by_id[live_view_data["id"]].live_view.update(live_view_data)

    # Global
    def get_info(self):
        """Return general informations about the Surveillance Station instance."""
        return self._dsm.get(self.INFO_API_KEY, "GetInfo")

    # Camera
    def get_all_cameras(self):
        """Return a list of cameras."""
        return self._cameras_by_id.values()

    def get_camera(self, camera_id):
        """Return camera matching camera_id."""
        return self._cameras_by_id[camera_id]

    def get_camera_live_view_path(self, camera_id, video_format=None):
        """Return camera live view path matching camera_id.

        Args:
            camera_id: ID of the camera we want to get the live view path.
            video_format: mjpeg_http | multicast | mxpeg_http |  rtsp_http | rtsp.
        """
        if video_format:
            return getattr(self._cameras_by_id[camera_id].live_view, video_format)
        return self._cameras_by_id[camera_id].live_view

    def get_camera_image(self, camera_id, profile=SNAPSHOT_PROFILE_BALANCED):
        """Return bytes of camera image for camera matching camera_id.

        Args:
            camera_id: ID of the camera we want to take a snapshot from
            profile: SNAPSHOT_PROFILE_HIGH_QUALITY |
                     SNAPSHOT_PROFILE_BALANCED |
                     SNAPSHOT_PROFILE_LOW_BANDWIDTH
        """
        return self._dsm.get(
            self.CAMERA_API_KEY,
            "GetSnapshot",
            {"id": camera_id, "cameraId": camera_id, "profileType": profile},
        )

    def enable_camera(self, camera_id):
        """Enable camera(s) - multiple ID or single ex 1 or 1,2,3."""
        return self._dsm.get(self.CAMERA_API_KEY, "Enable", {"idList": camera_id})[
            "success"
        ]

    def disable_camera(self, camera_id):
        """Disable camera(s) - multiple ID or single ex 1 or 1,2,3."""
        return self._dsm.get(self.CAMERA_API_KEY, "Disable", {"idList": camera_id})[
            "success"
        ]

    # Snapshot
    def capture_camera_image(self, camera_id, save=True):
        """Capture a snapshot for camera matching camera_id."""
        return self._dsm.get(
            self.SNAPSHOT_API_KEY,
            "TakeSnapshot",
            {
                "camId": camera_id,
                "blSave": int(save),  # API requires an integer instead of a boolean
            },
        )

    def download_snapshot(self, snapshot_id, snapshot_size):
        """Download snapshot image binary for a givent snapshot_id.

        Args:
            snapshot_id: ID of the snapshot we want to download.
            snapshot_size: SNAPSHOT_SIZE_ICON | SNAPSHOT_SIZE_FULL.
        """
        return self._dsm.get(
            self.SNAPSHOT_API_KEY,
            "LoadSnapshot",
            {"id": snapshot_id, "imgSize": snapshot_size},
        )

    # Motion
    def is_motion_detection_enabled(self, camera_id):
        """Return motion setting matching camera_id."""
        return self._cameras_by_id[camera_id].is_motion_detection_enabled

    def enable_motion_detection(self, camera_id):
        """Enable motion detection for camera matching camera_id."""
        return self._dsm.get(
            self.CAMERA_EVENT_API_KEY,
            "MDParamSave",
            {"camId": camera_id, "source": MOTION_DETECTION_BY_SURVEILLANCE},
        )

    def disable_motion_detection(self, camera_id):
        """Disable motion detection for camera matching camera_id."""
        return self._dsm.get(
            self.CAMERA_EVENT_API_KEY,
            "MDParamSave",
            {"camId": camera_id, "source": MOTION_DETECTION_DISABLED},
        )

    # Home mode
    def get_home_mode_status(self):
        """Get the state of Home Mode."""
        return self._dsm.get(self.HOME_MODE_API_KEY, "GetInfo")["data"]["on"]

    def set_home_mode(self, state):
        """Set the state of Home Mode (state: bool)."""
        return self._dsm.get(
            self.HOME_MODE_API_KEY, "Switch", {"on": str(state).lower()}
        )["success"]



"""DSM System data and actions."""


class SynoCoreSystem:
    """Class containing System data and actions."""

    API_KEY = "SYNO.Core.System"

    def __init__(self, dsm):
        """Constructor method."""
        self._dsm = dsm
        self._data = {}

    def update(self):
        """Updates System data."""
        raw_data = self._dsm.get(self.API_KEY, "info")
        if raw_data:
            self._data = raw_data["data"]

    #
    # get information
    #
    @property
    def cpu_clock_speed(self):
        """Gets System CPU clock speed."""
        return self._data.get("cpu_clock_speed")

    @property
    def cpu_cores(self):
        """Gets System CPU cores."""
        return self._data.get("cpu_cores")

    @property
    def cpu_family(self):
        """Gets System CPU family."""
        return self._data.get("cpu_family")

    @property
    def cpu_series(self):
        """Gets System CPU series."""
        return self._data.get("cpu_series")

    @property
    def enabled_ntp(self):
        """Gets System NTP state."""
        return self._data.get("enabled_ntp")

    @property
    def ntp_server(self):
        """Gets System NTP server."""
        return self._data.get("ntp_server")

    @property
    def firmware_ver(self):
        """Gets System firmware version."""
        return self._data.get("firmware_ver")

    @property
    def model(self):
        """Gets System model."""
        return self._data.get("model")

    @property
    def ram_size(self):
        """Gets System ram size."""
        return self._data.get("ram_size")

    @property
    def serial(self):
        """Gets System serial number."""
        return self._data.get("serial")

    @property
    def sys_temp(self):
        """Gets System temperature."""
        return self._data.get("sys_temp")

    @property
    def time(self):
        """Gets System time."""
        return self._data.get("time")

    @property
    def time_zone(self):
        """Gets System time zone."""
        return self._data.get("time_zone")

    @property
    def time_zone_desc(self):
        """Gets System time zone description."""
        return self._data.get("time_zone_desc")

    @property
    def up_time(self):
        """Gets System uptime."""
        return self._data.get("up_time")

    @property
    def usb_dev(self):
        """Gets System connected usb devices."""
        return self._data.get("usb_dev", [])

    #
    # do system actions
    #
    def shutdown(self):
        """Shutdown NAS."""
        res = self._dsm.get(
            self.API_KEY,
            "shutdown",
            max_version=1,  # shutdown method is only available on api version 1
        )
        return res

    def reboot(self):
        """Reboot NAS."""
        res = self._dsm.get(
            self.API_KEY,
            "reboot",
            max_version=1,  # reboot method is only available on api version 1
        )
        return res



"""DSM Network data."""


class SynoDSMNetwork:
    """Class containing Network data."""

    API_KEY = "SYNO.DSM.Network"

    def __init__(self, dsm):
        """Constructor method."""
        self._dsm = dsm
        self._data = {}

    def update(self):
        """Updates network data."""
        raw_data = self._dsm.get(self.API_KEY, "list")
        if raw_data:
            self._data = raw_data["data"]

    @property
    def dns(self):
        """DNS of the NAS."""
        return self._data.get("dns")

    @property
    def gateway(self):
        """Gateway of the NAS."""
        return self._data.get("gateway")

    @property
    def hostname(self):
        """Host name of the NAS."""
        return self._data.get("hostname")

    @property
    def interfaces(self):
        """Interfaces of the NAS."""
        return self._data.get("interfaces", [])

    def interface(self, eth_id):
        """Interface of the NAS."""
        for interface in self.interfaces:
            if interface["id"] == eth_id:
                return interface
        return None

    @property
    def macs(self):
        """MACs of the NAS."""  # noqa: D403
        macs = []
        for interface in self.interfaces:
            if interface.get("mac"):
                macs.append(interface["mac"])
        return macs

    @property
    def workgroup(self):
        """Workgroup of the NAS."""
        return self._data.get("workgroup")



"""DSM Upgrade data and actions."""


class SynoCoreUpgrade:
    """Class containing upgrade data and actions."""

    API_KEY = "SYNO.Core.Upgrade"
    API_SERVER_KEY = API_KEY + ".Server"

    def __init__(self, dsm):
        """Constructor method."""
        self._dsm = dsm
        self._data = {}

    def update(self):
        """Updates Upgrade data."""
        raw_data = self._dsm.get(self.API_SERVER_KEY, "check")
        if raw_data:
            self._data = raw_data["data"].get("update", raw_data["data"])

    @property
    def update_available(self):
        """Gets available update info."""
        return self._data.get("available")

    @property
    def available_version(self):
        """Gets available verion info."""
        return self._data.get("version")

    @property
    def reboot_needed(self):
        """Gets info if reboot is needed."""
        return self._data.get("reboot")

    @property
    def service_restarts(self):
        """Gets info if services are restarted."""
        return self._data.get("restart")



"""Synology DownloadStation API wrapper."""
from .task import SynoDownloadTask


class SynoDownloadStation:
    """An implementation of a Synology DownloadStation."""

    API_KEY = "SYNO.DownloadStation.*"
    INFO_API_KEY = "SYNO.DownloadStation.Info"
    STAT_API_KEY = "SYNO.DownloadStation.Statistic"
    TASK_API_KEY = "SYNO.DownloadStation.Task"

    def __init__(self, dsm):
        """Initialize a Download Station."""
        self._dsm = dsm
        self._tasks_by_id = {}
        self.additionals = [
            "detail",
            "file",
        ]  # Can contain: detail, transfer, file, tracker, peer

    def update(self):
        """Update tasks from API."""
        self._tasks_by_id = {}
        list_data = self._dsm.get(
            self.TASK_API_KEY, "List", {"additional": ",".join(self.additionals)}
        )["data"]
        for task_data in list_data["tasks"]:
            if task_data["id"] in self._tasks_by_id:
                self._tasks_by_id[task_data["id"]].update(task_data)
            else:
                self._tasks_by_id[task_data["id"]] = SynoDownloadTask(task_data)

    # Global
    def get_info(self):
        """Return general informations about the Download Station instance."""
        return self._dsm.get(self.INFO_API_KEY, "GetInfo")

    def get_config(self):
        """Return configuration about the Download Station instance."""
        return self._dsm.get(self.INFO_API_KEY, "GetConfig")

    def get_stat(self):
        """Return statistic about the Download Station instance."""
        return self._dsm.get(self.STAT_API_KEY, "GetInfo")

    # Downloads
    def get_all_tasks(self):
        """Return a list of tasks."""
        return self._tasks_by_id.values()

    def get_task(self, task_id):
        """Return task matching task_id."""
        return self._tasks_by_id[task_id]

    def create(self, uri, unzip_password=None, destination=None):
        """Create a new task (uri accepts HTTP/FTP/magnet/ED2K links)."""
        res = self._dsm.post(
            self.TASK_API_KEY,
            "Create",
            {
                "uri": ",".join(uri) if isinstance(uri, list) else uri,
                "unzip_password": unzip_password,
                "destination": destination,
            },
        )
        self.update()
        return res

    def pause(self, task_id):
        """Pause a download task."""
        res = self._dsm.get(
            self.TASK_API_KEY,
            "Pause",
            {"id": ",".join(task_id) if isinstance(task_id, list) else task_id},
        )
        self.update()
        return res

    def resume(self, task_id):
        """Resume a paused download task."""
        res = self._dsm.get(
            self.TASK_API_KEY,
            "Resume",
            {"id": ",".join(task_id) if isinstance(task_id, list) else task_id},
        )
        self.update()
        return res

    def delete(self, task_id, force_complete=False):
        """Delete a download task."""
        res = self._dsm.get(
            self.TASK_API_KEY,
            "Delete",
            {
                "id": ",".join(task_id) if isinstance(task_id, list) else task_id,
                "force_complete": force_complete,
            },
        )
        self.update()
        return res



"""DSM Security data."""


class SynoCoreSecurity:
    """Class containing Security data."""

    API_KEY = "SYNO.Core.SecurityScan.Status"

    def __init__(self, dsm):
        """Constructor method."""
        self._dsm = dsm
        self._data = {}

    def update(self):
        """Updates security data."""
        raw_data = self._dsm.get(self.API_KEY, "system_get")
        if raw_data:
            self._data = raw_data["data"]

    @property
    def checks(self):
        """Gets the checklist by check category."""
        return self._data.get("items", {})

    @property
    def last_scan_time(self):
        """Gets the last scan time."""
        return self._data.get("lastScanTime")

    @property
    def start_time(self):
        """Gets the start time (if in progress)."""
        return self._data.get("startTime")

    @property
    def success(self):
        """Gets the last scan success."""
        return self._data.get("success")

    @property
    def progress(self):
        """Gets the scan progress.

        Returns: 100 if finished
        """
        return self._data.get("sysProgress")

    @property
    def status(self):
        """Gets the last scan status.

        Possible values: safe, danger, info, outOfDate, risk, warning.
        """
        return self._data.get("sysStatus")

    @property
    def status_by_check(self):
        """Gets the last scan status per check."""
        status = {}
        for category in self.checks:
            status[category] = self.checks[category]["failSeverity"]
        return status



"""Class to interact with Synology DSM."""
import logging
import socket
from json import JSONDecodeError
from urllib.parse import quote

import urllib3
from requests import Session
from requests.exceptions import RequestException

from .api.core.security import SynoCoreSecurity
from .api.core.share import SynoCoreShare
from .api.core.system import SynoCoreSystem
from .api.core.upgrade import SynoCoreUpgrade
from .api.core.utilization import SynoCoreUtilization
from .api.download_station import SynoDownloadStation
from .api.dsm.information import SynoDSMInformation
from .api.dsm.network import SynoDSMNetwork
from .api.storage.storage import SynoStorage
from .api.surveillance_station import SynoSurveillanceStation
from .const import API_AUTH
from .const import API_INFO
from .const import SENSITIV_PARAMS
from .exceptions import SynologyDSMAPIErrorException
from .exceptions import SynologyDSMAPINotExistsException
from .exceptions import SynologyDSMLogin2SAFailedException
from .exceptions import SynologyDSMLogin2SAForcedException
from .exceptions import SynologyDSMLogin2SARequiredException
from .exceptions import SynologyDSMLoginDisabledAccountException
from .exceptions import SynologyDSMLoginFailedException
from .exceptions import SynologyDSMLoginInvalidException
from .exceptions import SynologyDSMLoginPermissionDeniedException
from .exceptions import SynologyDSMRequestException

_LOGGER = logging.getLogger(__name__)


class SynologyDSM:
    """Class containing the main Synology DSM functions."""

    DSM_5_WEIRD_URL_API = [
        SynoStorage.API_KEY,
    ]

    def __init__(
        self,
        dsm_ip: str,
        dsm_port: int,
        username: str,
        password: str,
        use_https: bool = False,
        verify_ssl: bool = False,
        timeout: int = None,
        device_token: str = None,
        debugmode: bool = False,
    ):
        """Constructor method."""
        self.username = username
        self._password = password
        self._timeout = timeout or 10
        self._debugmode = debugmode
        self._verify = verify_ssl & use_https

        # Session
        self._session = Session()
        self._session.verify = self._verify

        # Login
        self._session_id = None
        self._syno_token = None
        self._device_token = device_token

        # Services
        self._apis = {
            "SYNO.API.Info": {"maxVersion": 1, "minVersion": 1, "path": "query.cgi"}
        }
        self._download = None
        self._information = None
        self._network = None
        self._security = None
        self._share = None
        self._storage = None
        self._surveillance = None
        self._system = None
        self._utilisation = None
        self._upgrade = None

        # Build variables
        if use_https:
            if not verify_ssl:
                # https://urllib3.readthedocs.io/en/latest/advanced-usage.html#ssl-warnings  # noqa: B950
                # disable SSL warnings due to the auto-genenerated cert
                urllib3.disable_warnings()

            self._base_url = f"https://{dsm_ip}:{dsm_port}"
        else:
            self._base_url = f"http://{dsm_ip}:{dsm_port}"

    def _debuglog(self, message: str):
        """Outputs message if debug mode is enabled."""
        _LOGGER.debug(message)
        if self._debugmode:
            print("DEBUG: " + message)

    def _is_weird_api_url(self, api: str) -> bool:
        """Returns True if the API URL is not common.

        Common template is nas_base_url/webapi/path?params
        Only handles DSM 5 for now.
        """
        return (
            api in self.DSM_5_WEIRD_URL_API
            and self._information
            and self._information.version
            and int(self._information.version) < 7321  # < DSM 6
        )

    def _build_url(self, api: str) -> str:
        if self._is_weird_api_url(api):
            if api == SynoStorage.API_KEY:
                return (
                    f"{self._base_url}/webman/modules/StorageManager/"
                    f"storagehandler.cgi?"
                )

        return f"{self._base_url}/webapi/{self.apis[api]['path']}?"

    def discover_apis(self):
        """Retreives available API infos from the NAS."""
        if self._apis.get(API_AUTH):
            return
        self._apis = self.get(API_INFO, "query")["data"]

    @property
    def apis(self):
        """Gets available API infos from the NAS."""
        return self._apis

    def login(self, otp_code: str = None) -> bool:
        """Create a logged session."""
        # First reset the session
        self._debuglog("Creating new session")
        self._session = Session()
        self._session.verify = self._verify

        params = {
            "account": self.username,
            "passwd": self._password,
            # "enable_syno_token": "yes",
            "enable_device_token": "yes",
            "device_name": socket.gethostname(),
            "format": "sid",
        }

        if otp_code:
            params["otp_code"] = otp_code
        if self._device_token:
            params["device_id"] = self._device_token

        # Request login
        result = self.get(API_AUTH, "login", params)

        # Handle errors
        if result.get("error"):
            switcher = {
                400: SynologyDSMLoginInvalidException(self.username),
                401: SynologyDSMLoginDisabledAccountException(self.username),
                402: SynologyDSMLoginPermissionDeniedException(self.username),
                403: SynologyDSMLogin2SARequiredException(self.username),
                404: SynologyDSMLogin2SAFailedException(),
                406: SynologyDSMLogin2SAForcedException(self.username),
            }
            raise switcher.get(
                result["error"]["code"],
                SynologyDSMLoginFailedException(result["error"]["code"], self.username),
            )

        # Parse result if valid
        self._session_id = result["data"]["sid"]
        if result["data"].get("synotoken"):
            # Not available on API version < 3
            self._syno_token = result["data"]["synotoken"]
        if result["data"].get("did"):
            # Not available on API version < 6 && device token is given once
            # per device_name
            self._device_token = result["data"]["did"]
        if result["data"].get("device_id"):
            # Not available on API version < 7
            self._device_token = result["data"]["device_id"]
        self._debuglog("Authentication successful, token: " + str(self._session_id))

        if not self._information:
            self._information = SynoDSMInformation(self)
            self._information.update()

        return result["success"]

    def logout(self) -> bool:
        """Log out of the session."""
        result = self.get(API_AUTH, "logout")
        self._session = None
        return result["success"]

    @property
    def device_token(self) -> str:
        """Gets the device token.

        Used to remember the 2SA access was granted on this device.
        """
        return self._device_token

    def get(self, api: str, method: str, params: dict = None, **kwargs):
        """Handles API GET request."""
        return self._request("GET", api, method, params, **kwargs)

    def post(self, api: str, method: str, params: dict = None, **kwargs):
        """Handles API POST request."""
        return self._request("POST", api, method, params, **kwargs)

    def _request(
        self,
        request_method: str,
        api: str,
        method: str,
        params: dict = None,
        retry_once: bool = True,
        **kwargs,
    ):
        """Handles API request."""
        # Discover existing APIs
        if api != API_INFO:
            self.discover_apis()

        # Check if logged
        if not self._session_id and api not in [API_AUTH, API_INFO]:
            self.login()

        # Build request params
        if not params:
            params = {}
        params["api"] = api
        params["version"] = 1

        if not self._is_weird_api_url(api):
            # Check if API is available
            if not self.apis.get(api):
                raise SynologyDSMAPINotExistsException(api)
            params["version"] = self.apis[api]["maxVersion"]
            max_version = kwargs.pop("max_version", None)
            if max_version and params["version"] > max_version:
                params["version"] = max_version

        params["method"] = method

        if api == SynoStorage.API_KEY:
            params["action"] = method
        if self._session_id:
            params["_sid"] = self._session_id
        if self._syno_token:
            params["SynoToken"] = self._syno_token

        url = self._build_url(api)

        # Request data
        response = self._execute_request(request_method, url, params, **kwargs)
        self._debuglog("Request Method: " + request_method)
        self._debuglog("Successful returned data")
        self._debuglog("API: " + api)
        self._debuglog("RESPONSE: " + str(response))

        # Handle data errors
        if isinstance(response, dict) and response.get("error") and api != API_AUTH:
            self._debuglog("Session error: " + str(response["error"]["code"]))
            if response["error"]["code"] == 119 and retry_once:
                # Session ID not valid
                # see https://github.com/aerialls/synology-srm/pull/3
                self._session_id = None
                self._syno_token = None
                self._device_token = None
                return self._request(request_method, api, method, params, False)
            raise SynologyDSMAPIErrorException(
                api, response["error"]["code"], response["error"].get("errors")
            )

        return response

    def _execute_request(self, method: str, url: str, params: dict, **kwargs):
        """Function to execute and handle a request."""
        # Execute Request
        try:
            if method == "GET":
                encoded_params = "&".join(
                    f"{key}={quote(str(value))}" for key, value in params.items()
                )
                response = self._session.get(
                    url, params=encoded_params, timeout=self._timeout, **kwargs
                )
            elif method == "POST":
                data = {}
                data.update(params)
                data.update(kwargs.pop("data", {}))
                data["mimeType"] = "application/json"
                kwargs["data"] = data
                self._debuglog("POST data: " + str(data))

                response = self._session.post(
                    url, params=params, timeout=self._timeout, **kwargs
                )

            response_url = response.url
            for param in SENSITIV_PARAMS:
                if params.get(param):
                    response_url = response_url.replace(
                        quote(params[param]), "********"
                    )
            self._debuglog("Request url: " + response_url)
            self._debuglog("Request status_code: " + str(response.status_code))
            self._debuglog("Request headers: " + str(response.headers))

            if response.status_code == 200:
                # We got a DSM response
                content_type = response.headers.get("Content-Type", "").split(";")[0]

                if content_type in [
                    "application/json",
                    "text/json",
                    "text/plain",  # Can happen with some API
                ]:
                    return response.json()

                return response.content

            # We got a 400, 401 or 404 ...
            raise RequestException(response)

        except (RequestException, JSONDecodeError) as exp:
            raise SynologyDSMRequestException(exp) from exp

    def update(self, with_information: bool = False, with_network: bool = False):
        """Updates the various instanced modules."""
        if self._download:
            self._download.update()

        if self._information and with_information:
            self._information.update()

        if self._network and with_network:
            self._network.update()

        if self._security:
            self._security.update()

        if self._utilisation:
            self._utilisation.update()

        if self._storage:
            self._storage.update()

        if self._share:
            self._share.update()

        if self._surveillance:
            self._surveillance.update()

        if self._system:
            self._system.update()

        if self._upgrade:
            self._upgrade.update()

    def reset(self, api: any) -> bool:
        """Reset an API to avoid fetching in on update."""
        if isinstance(api, str):
            if api in ("information", SynoDSMInformation.API_KEY):
                return False
            if hasattr(self, "_" + api):
                setattr(self, "_" + api, None)
                return True
            if api == SynoCoreSecurity.API_KEY:
                self._security = None
                return True
            if api == SynoCoreShare.API_KEY:
                self._share = None
                return True
            if api == SynoCoreSystem.API_KEY:
                self._system = None
                return True
            if api == SynoCoreUpgrade.API_KEY:
                self._upgrade = None
                return True
            if api == SynoCoreUtilization.API_KEY:
                self._utilisation = None
                return True
            if api == SynoDownloadStation.API_KEY:
                self._download = None
                return True
            if api == SynoStorage.API_KEY:
                self._storage = None
                return True
            if api == SynoSurveillanceStation.API_KEY:
                self._surveillance = None
                return True
        if isinstance(api, SynoCoreSecurity):
            self._security = None
            return True
        if isinstance(api, SynoCoreShare):
            self._share = None
            return True
        if isinstance(api, SynoCoreSystem):
            self._system = None
            return True
        if isinstance(api, SynoCoreUpgrade):
            self._upgrade = None
            return True
        if isinstance(api, SynoCoreUtilization):
            self._utilisation = None
            return True
        if isinstance(api, SynoDownloadStation):
            self._download = None
            return True
        if isinstance(api, SynoStorage):
            self._storage = None
            return True
        if isinstance(api, SynoSurveillanceStation):
            self._surveillance = None
            return True
        return False

    @property
    def download_station(self) -> SynoDownloadStation:
        """Gets NAS DownloadStation."""
        if not self._download:
            self._download = SynoDownloadStation(self)
        return self._download

    @property
    def information(self) -> SynoDSMInformation:
        """Gets NAS informations."""
        if not self._information:
            self._information = SynoDSMInformation(self)
        return self._information

    @property
    def network(self) -> SynoDSMNetwork:
        """Gets NAS network informations."""
        if not self._network:
            self._network = SynoDSMNetwork(self)
        return self._network

    @property
    def security(self) -> SynoCoreSecurity:
        """Gets NAS security informations."""
        if not self._security:
            self._security = SynoCoreSecurity(self)
        return self._security

    @property
    def share(self) -> SynoCoreShare:
        """Gets NAS shares information."""
        if not self._share:
            self._share = SynoCoreShare(self)
        return self._share

    @property
    def storage(self) -> SynoStorage:
        """Gets NAS storage informations."""
        if not self._storage:
            self._storage = SynoStorage(self)
        return self._storage

    @property
    def surveillance_station(self) -> SynoSurveillanceStation:
        """Gets NAS SurveillanceStation."""
        if not self._surveillance:
            self._surveillance = SynoSurveillanceStation(self)
        return self._surveillance

    @property
    def system(self) -> SynoCoreSystem:
        """Gets NAS system information."""
        if not self._system:
            self._system = SynoCoreSystem(self)
        return self._system

    @property
    def upgrade(self) -> SynoCoreUpgrade:
        """Gets NAS upgrade informations."""
        if not self._upgrade:
            self._upgrade = SynoCoreUpgrade(self)
        return self._upgrade

    @property
    def utilisation(self) -> SynoCoreUtilization:
        """Gets NAS utilisation informations."""
        if not self._utilisation:
            self._utilisation = SynoCoreUtilization(self)
        return self._utilisation



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['synology_dsm',
 'synology_dsm.api',
 'synology_dsm.api.core',
 'synology_dsm.api.download_station',
 'synology_dsm.api.dsm',
 'synology_dsm.api.storage',
 'synology_dsm.api.surveillance_station']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.24.0,<3.0.0', 'urllib3>=1.25.10,<2.0.0']

entry_points = \
{'console_scripts': ['synologydsm-api = synology_dsm.__main__:main']}

setup_kwargs = {
    'name': 'py-synologydsm-api',
    'version': '1.0.5',
    'description': 'Python API for communication with Synology DSM',
    'long_description': '===========================\nPython API for Synology DSM\n===========================\n\n.. image:: https://github.com/mib1185/py-synologydsm-api/workflows/Tests/badge.svg\n    :target: https://github.com/mib1185/py-synologydsm-api/actions?query=workflow%3ATests+branch%3Amaster\n\n.. image:: https://img.shields.io/pypi/v/synologydsm-api.svg\n    :alt: Library version\n    :target: https://pypi.org/project/py-synologydsm-api\n\n.. image:: https://img.shields.io/pypi/pyversions/synologydsm-api.svg\n    :alt: Supported versions\n    :target: https://pypi.org/project/py-synologydsm-api\n\n.. image:: https://pepy.tech/badge/synologydsm-api\n    :alt: Downloads\n    :target: https://pypi.org/project/py-synologydsm-api\n\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n    :alt: Formated with Black\n    :target: https://github.com/psf/black\n\n\nInstallation\n============\n\n.. code-block:: bash\n\n    [sudo] pip install py-synologydsm-api\n\n\nUsage\n=====\n\nYou can import the module as `synology_dsm`.\n\n\nConstructor\n-----------\n\n.. code-block:: python\n\n    SynologyDSM(\n        dsm_ip,\n        dsm_port,\n        username,\n        password,\n        use_https=False,\n        verify_ssl=False,\n        timeout=None,\n        device_token=None,\n        debugmode=False,\n    )\n\n``device_token`` should be added when using a two-step authentication account, otherwise DSM will ask to login with a One Time Password (OTP) and requests will fail (see the login section for more details).\n\nDefault ``timeout`` is 10 seconds.\n\n\nLogin\n------\n\nThe library automatically login at first request, but you better use the ``login()`` function separately to authenticate.\n\nIt will return a boolean if it successed or faild to authenticate to DSM.\n\nIf your account need a two-step authentication (2SA), ``login()`` will raise ``SynologyDSMLogin2SARequiredException``.\nCall the function again with a One Time Password (OTP) as parameter, like ``login("123456")`` (better to be a string to handle first zero).\nStore the ``device_token`` property so that you do not need to reconnect with password the next time you open a new ``SynologyDSM`` session.\n\n\nCode exemple\n------------\n\nEvery API has an ``update()`` function that is needed to get the first data, then the data is cached and updated at the next ``update()`` call.\n\nThe ``SynologyDSM`` class can also ``update()`` all APIs at once.\n\n.. code-block:: python\n\n    from synology_dsm import SynologyDSM\n\n    print("Creating Valid API")\n    api = SynologyDSM("<IP/DNS>", "<port>", "<username>", "<password>")\n\n    print("=== Information ===")\n    api.information.update()\n    print("Model:           " + str(api.information.model))\n    print("RAM:             " + str(api.information.ram) + " MB")\n    print("Serial number:   " + str(api.information.serial))\n    print("Temperature:     " + str(api.information.temperature) + " °C")\n    print("Temp. warning:   " + str(api.information.temperature_warn))\n    print("Uptime:          " + str(api.information.uptime))\n    print("Full DSM version:" + str(api.information.version_string))\n    print("--")\n\n    print("=== Utilisation ===")\n    api.utilisation.update()\n    print("CPU Load:        " + str(api.utilisation.cpu_total_load) + " %")\n    print("Memory Use:      " + str(api.utilisation.memory_real_usage) + " %")\n    print("Net Up:          " + str(api.utilisation.network_up()))\n    print("Net Down:        " + str(api.utilisation.network_down()))\n    print("--")\n\n    print("=== Storage ===")\n    api.storage.update()\n    for volume_id in api.storage.volumes_ids:\n        print("ID:          " + str(volume_id))\n        print("Status:      " + str(api.storage.volume_status(volume_id)))\n        print("% Used:      " + str(api.storage.volume_percentage_used(volume_id)) + " %")\n        print("--")\n\n    for disk_id in api.storage.disks_ids:\n        print("ID:          " + str(disk_id))\n        print("Name:        " + str(api.storage.disk_name(disk_id)))\n        print("S-Status:    " + str(api.storage.disk_smart_status(disk_id)))\n        print("Status:      " + str(api.storage.disk_status(disk_id)))\n        print("Temp:        " + str(api.storage.disk_temp(disk_id)))\n        print("--")\n\n    print("=== Shared Folders ===")\n    api.share.update()\n    for share_uuid in api.share.shares_uuids:\n        print("Share name:        " + str(api.share.share_name(share_uuid)))\n        print("Share path:        " + str(api.share.share_path(share_uuid)))\n        print("Space used:        " + str(api.share.share_size(share_uuid, human_readable=True)))\n        print("Recycle Bin Enabled: " + str(api.share.share_recycle_bin(share_uuid)))\n        print("--")\n\n\nDownload Station usage\n--------------------------\n\n.. code-block:: python\n\n    from synology_dsm import SynologyDSM\n\n    api = SynologyDSM("<IP/DNS>", "<port>", "<username>", "<password>")\n\n    if "SYNO.DownloadStation.Info" in api.apis:\n\n        api.download_station.get_info()\n        api.download_station.get_config()\n\n        # The download list will be updated after each of the following functions:\n        # You should have the right on the (default) directory that the download will be saved, or you will get a 403 or 406 error\n        api.download_station.create("http://commondatastorage.googleapis.com/gtv-videos-bucket/sample/BigBuckBunny.mp4")\n        api.download_station.pause("dbid_1")\n        # Like the other function, you can eather pass a str or a list\n        api.download_station.resume(["dbid_1", "dbid_2"])\n        api.download_station.delete("dbid_3")\n\n        # Manual update\n        api.download_station.update()\n\n\nSurveillance Station usage\n--------------------------\n\n.. code-block:: python\n\n    from synology_dsm import SynologyDSM\n\n    api = SynologyDSM("<IP/DNS>", "<port>", "<username>", "<password>")\n    surveillance = api.surveillance_station\n    surveillance.update() # First update is required\n\n    # Returns a list of cached cameras available\n    cameras = surveillance.get_all_cameras()\n\n    # Assuming there\'s at least one camera, get the first camera_id\n    camera_id = cameras[0].camera_id\n\n    # Returns cached camera object by camera_id\n    camera = surveillance.get_camera(camera_id)\n\n    # Returns cached motion detection enabled\n    motion_setting = camera.is_motion_detection_enabled\n\n    # Return bytes of camera image\n    surveillance.get_camera_image(camera_id)\n\n    # Updates all cameras/motion settings and cahce them\n    surveillance.update()\n\n    # Gets Home Mode status\n    home_mode_status =  surveillance.get_home_mode_status()\n\n    # Sets home mode - true is on, false is off\n    surveillance.set_home_mode(True)\n\n\nSystem usage\n--------------------------\n\n.. code-block:: python\n\n    from synology_dsm import SynologyDSM\n\n    api = SynologyDSM("<IP/DNS>", "<port>", "<username>", "<password>")\n    system = api.system\n\n    # Reboot NAS\n    system.reboot()\n\n    # Shutdown NAS\n    system.shutdown()\n\n    # Manual update system information\n    system.update()\n\n    # Get CPU information\n    system.cpu_clock_speed\n    system.cpu_cores\n    system.cpu_family\n    system.cpu_series\n\n    # Get NTP settings\n    system.enabled_ntp\n    system.ntp_server\n\n    # Get system information\n    system.firmware_ver\n    system.model\n    system.ram_size\n    system.serial\n    system.sys_temp\n    system.time\n    system.time_zone\n    system.time_zone_desc\n    system.up_time\n\n    # Get list of all connected USB devices\n    system.usb_dev\n\n\nUpgrade usage\n--------------------------\n\n.. code-block:: python\n\n    from synology_dsm import SynologyDSM\n\n    api = SynologyDSM("<IP/DNS>", "<port>", "<username>", "<password>")\n    upgrade = api.upgrade\n\n    # Manual update upgrade information\n    upgrade.update()\n\n    # check if DSM update is available\n    if upgrade.update_available:\n        do something ...\n\n    # get available version string (return None if no update available)\n    upgrade.available_version\n\n    # get need of reboot (return None if no update available)\n    upgrade.reboot_needed\n\n    # get need of service restarts (return None if no update available)\n    upgrade.service_restarts\n\n\nCredits / Special Thanks\n========================\n- https://github.com/florianeinfalt\n- https://github.com/tchellomello\n- https://github.com/Quentame   (Multiple API addition & tests)\n- https://github.com/aaska      (DSM 5 tests)\n- https://github.com/chemelli74 (2SA tests)\n- https://github.com/snjoetw    (Surveillance Station library)\n- https://github.com/shenxn     (Surveillance Station tests)\n- https://github.com/Gestas     (Shared Folders)\n\nFound Synology API "documentation" on this repo : https://github.com/kwent/syno/tree/master/definitions\n\n\nOfficial references\n===================\n\n- `Calendar API documentation (2015-2019) <https://global.download.synology.com/download/Document/Software/DeveloperGuide/Package/Calendar/2.4/enu/Synology_Calendar_API_Guide_enu.pdf>`_\n\n- `Download Station API documentation (2012-2014) <https://global.download.synology.com/download/Document/Software/DeveloperGuide/Package/DownloadStation/All/enu/Synology_Download_Station_Web_API.pdf>`_\n\n- `File Station API documentation (2013-2019) <https://global.download.synology.com/download/Document/Software/DeveloperGuide/Package/FileStation/All/enu/Synology_File_Station_API_Guide.pdf>`_\n\n- `Surveillance Station API documentation (2012-2020) <https://global.download.synology.com/download/Document/Software/DeveloperGuide/Package/SurveillanceStation/All/enu/Surveillance_Station_Web_API.pdf>`_\n\n- `Virtual Machine Manager API documentation (2015-2019) <https://global.download.synology.com/download/Document/Software/DeveloperGuide/Package/Virtualization/All/enu/Synology_Virtual_Machine_Manager_API_Guide.pdf>`_\n',
    'author': 'Michael Brussk (mib1185)',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mib1185/py-synologydsm-api',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.0,<4.0.0',
}


setup(**setup_kwargs)



"""DSM Storage data."""
from synology_dsm.helpers import SynoFormatHelper


class SynoStorage:
    """Class containing Storage data."""

    API_KEY = "SYNO.Storage.CGI.Storage"

    def __init__(self, dsm):
        """Constructor method."""
        self._dsm = dsm
        self._data = {}

    def update(self):
        """Updates storage data."""
        raw_data = self._dsm.get(self.API_KEY, "load_info")
        if raw_data:
            self._data = raw_data
            if raw_data.get("data"):
                self._data = raw_data["data"]

    # Root
    @property
    def disks(self):
        """Gets all (internal) disks."""
        return self._data.get("disks", [])

    @property
    def env(self):
        """Gets storage env."""
        return self._data.get("env")

    @property
    def storage_pools(self):
        """Gets all storage pools."""
        return self._data.get("storagePools", [])

    @property
    def volumes(self):
        """Gets all volumes."""
        return self._data.get("volumes", [])

    # Volume
    @property
    def volumes_ids(self):
        """Returns volumes ids."""
        volumes = []
        for volume in self.volumes:
            volumes.append(volume["id"])
        return volumes

    def get_volume(self, volume_id):
        """Returns a specific volume."""
        for volume in self.volumes:
            if volume["id"] == volume_id:
                return volume
        return {}

    def volume_status(self, volume_id):
        """Status of the volume (normal, degraded, etc)."""
        return self.get_volume(volume_id).get("status")

    def volume_device_type(self, volume_id):
        """Returns the volume type (RAID1, RAID2, etc)."""
        return self.get_volume(volume_id).get("device_type")

    def volume_size_total(self, volume_id, human_readable=False):
        """Total size of volume."""
        volume = self.get_volume(volume_id)
        if volume.get("size"):
            return_data = int(volume["size"]["total"])
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None

    def volume_size_used(self, volume_id, human_readable=False):
        """Total used size in volume."""
        volume = self.get_volume(volume_id)
        if volume.get("size"):
            return_data = int(volume["size"]["used"])
            if human_readable:
                return SynoFormatHelper.bytes_to_readable(return_data)
            return return_data
        return None

    def volume_percentage_used(self, volume_id):
        """Total used size in percentage for volume."""
        volume = self.get_volume(volume_id)
        if volume.get("size"):
            total = int(volume["size"]["total"])
            used = int(volume["size"]["used"])

            if used and used > 0 and total and total > 0:
                return round((float(used) / float(total)) * 100.0, 1)
        return None

    def volume_disk_temp_avg(self, volume_id):
        """Average temperature of all disks making up the volume."""
        vol_disks = self._get_disks_for_volume(volume_id)
        if vol_disks:
            total_temp = 0
            total_disks = 0

            for vol_disk in vol_disks:
                disk_temp = self.disk_temp(vol_disk["id"])
                if disk_temp:
                    total_disks += 1
                    total_temp += disk_temp

            if total_temp > 0 and total_disks > 0:
                return round(total_temp / total_disks, 0)
        return None

    def volume_disk_temp_max(self, volume_id):
        """Maximum temperature of all disks making up the volume."""
        vol_disks = self._get_disks_for_volume(volume_id)
        if vol_disks:
            max_temp = 0

            for vol_disk in vol_disks:
                disk_temp = self.disk_temp(vol_disk["id"])
                if disk_temp and disk_temp > max_temp:
                    max_temp = disk_temp
            return max_temp
        return None

    # Disk
    @property
    def disks_ids(self):
        """Returns (internal) disks ids."""
        disks = []
        for disk in self.disks:
            disks.append(disk["id"])
        return disks

    def get_disk(self, disk_id):
        """Returns a specific disk."""
        for disk in self.disks:
            if disk["id"] == disk_id:
                return disk
        return {}

    def _get_disks_for_volume(self, volume_id):
        """Returns a list of disk for a specific volume."""
        disks = []
        for pool in self.storage_pools:

            if pool.get("deploy_path") == volume_id:
                # RAID disk redundancy
                for disk_id in pool["disks"]:
                    disks.append(self.get_disk(disk_id))

            if pool.get("pool_child"):
                # SHR disk redundancy
                for pool_child in pool.get("pool_child"):
                    if pool_child["id"] == volume_id:
                        for disk_id in pool["disks"]:
                            disks.append(self.get_disk(disk_id))

        return disks

    def disk_name(self, disk_id):
        """The name of this disk."""
        return self.get_disk(disk_id).get("name")

    def disk_device(self, disk_id):
        """The mount point of this disk."""
        return self.get_disk(disk_id).get("device")

    def disk_smart_status(self, disk_id):
        """Status of disk according to S.M.A.R.T)."""
        return self.get_disk(disk_id).get("smart_status")

    def disk_status(self, disk_id):
        """Status of disk."""
        return self.get_disk(disk_id).get("status")

    def disk_exceed_bad_sector_thr(self, disk_id):
        """Checks if disk has exceeded maximum bad sector threshold."""
        return self.get_disk(disk_id).get("exceed_bad_sector_thr")

    def disk_below_remain_life_thr(self, disk_id):
        """Checks if disk has fallen below minimum life threshold."""
        return self.get_disk(disk_id).get("below_remain_life_thr")

    def disk_temp(self, disk_id):
        """Returns the temperature of the disk."""
        return self.get_disk(disk_id).get("temp")



"""Synology SurveillanceStation API constants."""

RECORDING_STATUS = [
    1,  # Continue recording schedule
    2,  # Motion detect recording schedule
    3,  # Digital input recording schedule
    4,  # Digital input recording schedule
    5,  # Manual recording schedule
]
MOTION_DETECTION_DISABLED = -1
MOTION_DETECTION_BY_CAMERA = 0
MOTION_DETECTION_BY_SURVEILLANCE = 1

SNAPSHOT_SIZE_ICON = 1
SNAPSHOT_SIZE_FULL = 2

SNAPSHOT_PROFILE_HIGH_QUALITY = 0
SNAPSHOT_PROFILE_BALANCED = 1
SNAPSHOT_PROFILE_LOW_BANDWIDTH = 2



"""SurveillanceStation camera."""
from .const import MOTION_DETECTION_DISABLED
from .const import RECORDING_STATUS


class SynoCamera:
    """An representation of a Synology SurveillanceStation camera."""

    def __init__(self, data, live_view_data=None):
        """Initialize a Surveillance Station camera."""
        self._data = data
        self.live_view = SynoCameraLiveView(live_view_data)
        self._motion_detection_enabled = None

    def update(self, data):
        """Update the camera."""
        self._data = data

    def update_motion_detection(self, data):
        """Update the camera motion detection."""
        self._motion_detection_enabled = (
            MOTION_DETECTION_DISABLED != data["MDParam"]["source"]
        )

    @property
    def id(self):
        """Return id of the camera."""
        return self._data["id"]

    @property
    def name(self):
        """Return name of the camera."""
        return self._data["name"]

    @property
    def model(self):
        """Return model of the camera."""
        return self._data["model"]

    @property
    def resolution(self):
        """Return resolution of the camera."""
        return self._data["resolution"]

    @property
    def fps(self):
        """Return FPS of the camera."""
        return self._data["fps"]

    @property
    def is_enabled(self):
        """Return true if camera is enabled."""
        return self._data["enabled"]

    @property
    def is_motion_detection_enabled(self):
        """Return true if motion detection is enabled."""
        return self._motion_detection_enabled

    @property
    def is_recording(self):
        """Return true if camera is recording."""
        return self._data["recStatus"] in RECORDING_STATUS


class SynoCameraLiveView:
    """An representation of a Synology SurveillanceStation camera live view."""

    def __init__(self, data):
        """Initialize a Surveillance Station camera live view."""
        self.update(data)

    def update(self, data):
        """Update the camera live view."""
        self._data = data

    @property
    def mjpeg_http(self):
        """Return the mjpeg stream (over http) path of the camera."""
        return self._data["mjpegHttpPath"]

    @property
    def multicast(self):
        """Return the multi-cast path of the camera."""
        return self._data["multicstPath"]

    @property
    def mxpeg_http(self):
        """Return the mxpeg stream path of the camera."""
        return self._data["mxpegHttpPath"]

    @property
    def rtsp_http(self):
        """Return the RTSP stream (over http) path of the camera."""
        return self._data["rtspOverHttpPath"]

    @property
    def rtsp(self):
        """Return the RTSP stream path of the camera."""
        return self._data["rtspPath"]
